/**
 * 
 */
package edu.odu.cs.codeAnnotation;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * Annotates C++ & Java source code for inclusion in LaTeX documents
 * (using the listings package).
 * 
 * Annotations supported are:
 *     #...# => <:\smvdots:>  small vertical dots, the # stand for the slash-asterisk asterisk-slash comment markers
 *     //... => <:\smvdots:>
 *     #+N# => <+N>    These commands are repeated line-by-line, N an integer 1..3
 *     #-N# => <-N> 
 *     #coN# => <:\conum{n}:>   call-out numbers, N=1..9
 * @author zeil
 *
 */
public class Code2TeXSimple {
	
	BufferedReader input;
	BufferedWriter output;
	
	public Code2TeXSimple (String filename) throws IOException
	{
		input = new BufferedReader (new FileReader(filename));
		output = new BufferedWriter (new FileWriter(filename + ".tex"));
	}
	
	public Code2TeXSimple () throws IOException
	{
		input = null;
		output = null;
	}

	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		// takes source code file name as first parameter
		new Code2TeXSimple(args[0]).process();
	}

	/**
	 * Carry out substitutions across an entire file
	 * @throws IOException
	 */
	public void process() throws IOException {
		StringBuffer buf = new StringBuffer();
		String line;
		while ((line = input.readLine()) != null) {
			buf.append(line);
			buf.append("\n");
		}
		String result = process (buf.toString());
		output.write (result, 0, result.length());
		output.close();
	}

	/**
	 * Carry out substitutions within a multi-line string
	 * @param sourceCode
	 * @return soureCode with substitutions
	 */
	public String process(String sourceCode) {
		Pattern dots = Pattern.compile("/\\*\\**\\.\\.\\.\\*/");
		Matcher m = dots.matcher(sourceCode);
		while (m.find()) {
			String before = sourceCode.substring(0, m.start());
			String block = "<:\\smvdots:>";
			String after = sourceCode.substring(m.end());
			sourceCode = before + block + after;
			m = dots.matcher(sourceCode);
		}
		Pattern dots2 = Pattern.compile("//\\.\\.\\.");
		m = dots2.matcher(sourceCode);
		while (m.find()) {
			String before = sourceCode.substring(0, m.start());
			String block = "<:\\smvdots:>";
			String after = sourceCode.substring(m.end());
			sourceCode = before + block + after;
			m = dots.matcher(sourceCode);
		}
		
		Pattern callout = Pattern.compile("/\\*\\**co(\\d+)\\*/");
		m = callout.matcher(sourceCode);
		while (m.find()) {
			String before = sourceCode.substring(0, m.start());
			String block = "<:\\conum{" + m.group(1) + "}:>";
			String after = sourceCode.substring(m.end());
			sourceCode = before + block + after;
			m = dots.matcher(sourceCode);
		}
		
		
		Pattern plusBlock = Pattern.compile("(?s)/\\*\\+\\*/(.*?)/\\*\\**-\\*/");
		m = plusBlock.matcher(sourceCode);
		while (m.find()) {
			String before = sourceCode.substring(0, m.start());
			String block = m.group(1);
			String after = sourceCode.substring(m.end());
			block = block.replaceAll("\n", "<-1>\n<+1>");
			sourceCode = before + "<+1>" + block + "<-1>" + after;
			m = plusBlock.matcher(sourceCode);
		}
		
		plusBlock = Pattern.compile("(?s)/\\*\\+1\\*/(.*?)/\\*\\**-1\\*/");
		m = plusBlock.matcher(sourceCode);
		while (m.find()) {
			String before = sourceCode.substring(0, m.start());
			String block = m.group(1);
			String after = sourceCode.substring(m.end());
			block = block.replaceAll("\n", "<-1>\n<+1>");
			sourceCode = before + "<+1>" + block + "<-1>" + after;
			m = plusBlock.matcher(sourceCode);
		}
		
		plusBlock = Pattern.compile("(?s)/\\*\\+2\\*/(.*?)/\\*\\**-2\\*/");
		m = plusBlock.matcher(sourceCode);
		while (m.find()) {
			String before = sourceCode.substring(0, m.start());
			String block = m.group(1);
			String after = sourceCode.substring(m.end());
			block = block.replaceAll("\n", "<-2>\n<+2>");
			sourceCode = before + "<+2>" + block + "<-2>" + after;
			m = plusBlock.matcher(sourceCode);
		}

		
		plusBlock = Pattern.compile("(?s)/\\*\\+3\\*/(.*?)/\\*\\**-3\\*/");
		m = plusBlock.matcher(sourceCode);
		while (m.find()) {
			String before = sourceCode.substring(0, m.start());
			String block = m.group(1);
			String after = sourceCode.substring(m.end());
			block = block.replaceAll("\n", "<-3>\n<+3>");
			sourceCode = before + "<+3>" + block + "<-3>" + after;
			m = plusBlock.matcher(sourceCode);
		}

		return sourceCode;
	}

}
